/* Generated file (by generator) */

/*
 * SPDX-FileCopyrightText: 2015 William Yu <williamyu@gnome.org>
 *
 * SPDX-License-Identifier: LGPL-2.1-only OR MPL-2.0
 */

#ifndef LIBICAL_GLIB_H
#define LIBICAL_GLIB_H

#include <glib-object.h>

#define __LIBICAL_GLIB_H_INSIDE__

#include <libical-glib/i-cal-forward-declarations.h>
#include <libical-glib/i-cal-array.h>
#include <libical-glib/i-cal-attach.h>
#include <libical-glib/i-cal-comp-iter.h>
#include <libical-glib/i-cal-component.h>
#include <libical-glib/i-cal-datetimeperiod.h>
#include <libical-glib/i-cal-derived-parameter.h>
#include <libical-glib/i-cal-derived-property.h>
#include <libical-glib/i-cal-derived-value.h>
#include <libical-glib/i-cal-duration.h>
#include <libical-glib/i-cal-enum-array.h>
#include <libical-glib/i-cal-enums.h>
#include <libical-glib/i-cal-error.h>
#include <libical-glib/i-cal-geo.h>
#include <libical-glib/i-cal-limits.h>
#include <libical-glib/i-cal-memory.h>
#include <libical-glib/i-cal-param-iter.h>
#include <libical-glib/i-cal-parameter.h>
#include <libical-glib/i-cal-parser.h>
#include <libical-glib/i-cal-period.h>
#include <libical-glib/i-cal-prop-iter.h>
#include <libical-glib/i-cal-property.h>
#include <libical-glib/i-cal-recur-iterator.h>
#include <libical-glib/i-cal-recur.h>
#include <libical-glib/i-cal-recurrence.h>
#include <libical-glib/i-cal-reqstat.h>
#include <libical-glib/i-cal-restriction.h>
#include <libical-glib/i-cal-str-array.h>
#include <libical-glib/i-cal-time-span.h>
#include <libical-glib/i-cal-time.h>
#include <libical-glib/i-cal-timezone.h>
#include <libical-glib/i-cal-trigger.h>
#include <libical-glib/i-cal-unknowntokenhandling.h>
#include <libical-glib/i-cal-value.h>
#include <libical-glib/i-cal-vcard-component.h>
#include <libical-glib/i-cal-vcard-derived-parameter.h>
#include <libical-glib/i-cal-vcard-derived-property.h>
#include <libical-glib/i-cal-vcard-derived-value.h>
#include <libical-glib/i-cal-vcard-enum-array.h>
#include <libical-glib/i-cal-vcard-geo.h>
#include <libical-glib/i-cal-vcard-parameter.h>
#include <libical-glib/i-cal-vcard-parser.h>
#include <libical-glib/i-cal-vcard-property.h>
#include <libical-glib/i-cal-vcard-restriction.h>
#include <libical-glib/i-cal-vcard-str-array.h>
#include <libical-glib/i-cal-vcard-structured.h>
#include <libical-glib/i-cal-vcard-time.h>
#include <libical-glib/i-cal-vcard-tz.h>
#include <libical-glib/i-cal-vcard-value.h>

#undef __LIBICAL_GLIB_H_INSIDE__

#endif /* LIBICAL_GLIB_H */
